# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2022 Northwestern University.
#
# Invenio-Communities is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Entity resolver for the requests module.

Entity resolvers are considered part of the service-layer. The resolver
is registered in Invenio-Requests via the "invenio_requests.entity_resolvers"
entry point.
"""

from invenio_records_resources.references.resolvers.records import \
    RecordPKProxy, RecordResolver

from ..generators import CommunityRoleNeed
from .records.api import Community


class CommunityPKProxy(RecordPKProxy):
    """Resolver proxy for a Record entity using the UUID."""

    def get_needs(self, ctx=None):
        """Return community member need."""
        ctx = ctx or {}
        roles = ctx.get(
            # TODO: replace with COMMUNITIES_ROLES
            'community_roles', ['owner', 'manager', 'curator', 'reader']
        )
        comid = str(self._parse_ref_dict_id(self._ref_dict))
        return [CommunityRoleNeed(comid, role) for role in roles]


class CommunityResolver(RecordResolver):
    """Community entity resolver.

    The entity resolver enables Invenio-Requests to understand communities as
    receiver and topic of a request.
    """

    type_id = 'community'
    """Type identifier for this resolver."""

    def __init__(self):
        """Initialize the default record resolver."""
        super().__init__(
            Community, type_key=self.type_id, proxy_cls=CommunityPKProxy)

    def _reference_entity(self, entity):
        """Create a reference dict for the given record."""
        return {self.type_key: str(entity.id)}



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Utilities."""

from invenio_cache import current_cache

from .generators import CommunityRoleNeed
from .members.records.api import Member


def load_community_needs(identity):
    """Add community-related needs to the freshly loaded identity.

    Note that this function is intended to be called as handler for the
    identity-loaded signal, where we don't have control over the handler
    execution order.
    Thus, the given identity may not be fully initialized and still missing
    some needs (e.g. 'authenticated_user'), so we cannot rely on high-level
    service functions because permission checks may fail.
    """
    if identity.id is None:
        # no user is logged in
        return

    # Cache keys
    #
    # The cache of communities must be invalidated on:
    # 1) on creation of a community (likely this one is modelled via membership
    #    in the future).
    # 2) add/remove/change of membership
    #
    # We construct the cache key for each membership entity (e.g. user,
    # role, system role). This way, once a membership is added/removed/updated
    # we can cache the list of associated communities for this entity.
    # Once a user logs in, we get the cache for each of the membership
    # entities and combine it into a single list.

    # Currently, only users are supported (no roles or system roles)
    # cache_key = identity_cache_key(identity)
    # community_roles = current_cache.get(cache_key)
    # if community_roles is None:
    #         community_roles = Member.get_memberships(identity)
    #         current_cache.set(cache_key, community_roles, timeout=24*3600)

    # TODO: Fix caching issues

    # Add community needs to identity
    for c_id, role in Member.get_memberships(identity):
        identity.provides.add(CommunityRoleNeed(c_id, role))


def on_membership_change(identity=None):
    """Handler called when a membership is changed."""
    if identity is not None:
        current_cache.delete(identity_cache_key(identity))

def identity_cache_key(identity):
    """Make the cache key for storing the communities for a user."""
    return f"user-communities:{identity.id}"



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
# Copyright (C) 2022 Northwestern University.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Communities Service API config."""

from invenio_records_resources.services import FileServiceConfig
from invenio_records_resources.services.files.links import FileLink
from invenio_records_resources.services.records.components import \
    MetadataComponent, RelationsComponent
from invenio_records_resources.services.records.config import \
    RecordServiceConfig
from invenio_records_resources.services.records.config import \
    SearchOptions as SearchOptionsBase
from invenio_records_resources.services.records.links import RecordLink, \
    pagination_links

from invenio_communities.communities.records.api import Community
from invenio_communities.communities.services import facets
from invenio_communities.communities.services.results import CommunityFeaturedList

from ...permissions import CommunityPermissionPolicy
from ..schema import CommunityFeaturedSchema, CommunitySchema
from .components import CommunityAccessComponent, FeaturedCommunityComponent, OAISetComponent, \
    OwnershipComponent, PIDComponent

from flask_babelex import lazy_gettext as _


class SearchOptions(SearchOptionsBase):
    """Search options."""

    sort_options = {
        **SearchOptionsBase.sort_options,
        "featured": dict(
            title=_('Featured'),
            fields=[{"featured.past" : {"order" : "desc",}}],
        ),
    }

    facets = {
        'type': facets.type,
        'visibility': facets.visibility
    }


class CommunityMembersLink(RecordLink):
    """Link variables setter for Community Members links."""

    @staticmethod
    def vars(record, vars):
        """Variables for the URI template."""
        vars.update({"community_uuid": record.id})


class CommunityServiceConfig(RecordServiceConfig):
    """Communities service configuration."""
    service_id="communities"

    # Common configuration
    permission_policy_cls = CommunityPermissionPolicy

    # Record specific configuration
    record_cls = Community
    indexer_queue_name = "communities"

    # Search configuration
    search = SearchOptions

    # Service schema
    schema = CommunitySchema
    schema_featured = CommunityFeaturedSchema

    result_list_cls_featured = CommunityFeaturedList

    links_item = {
        "self": RecordLink("{+api}/communities/{id}"),
        "self_html": RecordLink("{+ui}/communities/{id}"),
        "settings_html": RecordLink("{+ui}/communities/{id}/settings"),
        "logo": RecordLink("{+api}/communities/{id}/logo"),
        "rename": RecordLink("{+api}/communities/{id}/rename"),
        "members": CommunityMembersLink("{+api}/communities/{community_uuid}/members"),
        "public_members": CommunityMembersLink("{+api}/communities/{community_uuid}/members/public"),
        "invitations": CommunityMembersLink("{+api}/communities/{community_uuid}/invitations")
    }

    links_search = pagination_links("{+api}/communities{?args*}")
    links_user_search = pagination_links("{+api}/user/communities{?args*}")

    # Service components
    components = [
        MetadataComponent,
        PIDComponent,
        RelationsComponent,
        CommunityAccessComponent,
        OwnershipComponent,
        FeaturedCommunityComponent,
        OAISetComponent,
    ]


class CommunityFileServiceConfig(FileServiceConfig):
    """Configuration for community files."""

    record_cls = Community
    permission_policy_cls = CommunityPermissionPolicy

    file_links_item = {
        "self": FileLink("{+api}/communities/{id}/logo"),
    }



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""API views."""

from flask import Blueprint

blueprint = Blueprint('invenio_communities_ext', __name__)


@blueprint.record_once
def init(state):
    """Init app."""
    app = state.app
    # Register services - cannot be done in extension because
    # Invenio-Records-Resources might not have been initialized.
    rr_ext = app.extensions['invenio-records-resources']
    idx_ext = app.extensions['invenio-indexer']
    ext = app.extensions['invenio-communities']

    # services
    rr_ext.registry.register(ext.service)
    # indexers
    idx_ext.registry.register(ext.service.indexer, indexer_id='communities')
    idx_ext.registry.register(
        ext.service.members.indexer, indexer_id='members'
    )
    # change notification handlers
    rr_ext.notification_registry.register(
        "users", ext.service.on_relation_update
    )


def create_communities_api_blueprint(app):
    """Create communities api blueprint."""
    ext = app.extensions["invenio-communities"]
    # control blueprint endpoints registration
    return ext.communities_resource.as_blueprint()


def create_members_api_bp_from_app(app):
    """Create members api blueprint."""
    ext = app.extensions["invenio-communities"]
    # control blueprint endpoints registration
    return ext.members_resource.as_blueprint()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

import itertools
from copy import deepcopy

import pytest
from flask_principal import AnonymousIdentity
from invenio_access.permissions import any_user as any_user_need,\
     superuser_access
from invenio_access.models import ActionRoles
from invenio_accounts.models import Role
from invenio_admin.permissions import action_admin_access
from invenio_app.factory import create_api
from invenio_requests.proxies import current_events_service, current_requests_service

from invenio_communities.communities.records.api import Community
from invenio_communities.proxies import current_communities
from invenio_vocabularies.contrib.affiliations.services import \
     AffiliationsService, AffiliationsServiceConfig
from invenio_vocabularies.proxies import current_service as vocabulary_service
from invenio_vocabularies.records.api import Vocabulary

pytest_plugins = ("celery.contrib.pytest", )


#
# Application
#
@pytest.fixture(scope='module')
def app_config(app_config):
    """Override pytest-invenio app_config fixture."""
    app_config['RECORDS_REFRESOLVER_CLS'] = \
        "invenio_records.resolver.InvenioRefResolver"
    app_config['RECORDS_REFRESOLVER_STORE'] = \
        "invenio_jsonschemas.proxies.current_refresolver_store"
    # Variable not used. We set it to silent warnings
    app_config['JSONSCHEMAS_HOST'] = 'not-used'

    return app_config


@pytest.fixture(scope="module")
def create_app(instance_path):
    """Application factory fixture."""
    return create_api


@pytest.fixture()
def headers():
    """Default headers for making requests."""
    return {
        'content-type': 'application/json',
        'accept': 'application/json',
    }


#
# Services
#
@pytest.fixture(scope="module")
def community_service(app):
    """Community service."""
    return current_communities.service


@pytest.fixture(scope="module")
def member_service(community_service):
    """Members subservice."""
    return community_service.members


@pytest.fixture(scope="module")
def requests_service(app):
    """Requests service."""
    return current_requests_service


@pytest.fixture(scope="module")
def events_service(app):
    """Requests service."""
    return current_events_service


@pytest.fixture(scope="module")
def vocabularies_service(app):
    """Vocabularies service."""
    return vocabulary_service


#
# Users and groups
#
@pytest.fixture(scope="module")
def anon_identity():
    """A new user."""
    identity = AnonymousIdentity()
    identity.provides.add(any_user_need)
    return identity


@pytest.fixture(scope="module")
def users(UserFixture, app, database):
    users = {}
    for r in ['owner', 'manager', 'curator', 'reader']:
        u = UserFixture(
            email=f'{r}@{r}.org',
            password=r,
        )
        u.create(app, database)
        users[r] = u
    return users


@pytest.fixture(scope="module")
def group(database):
    r = Role(name='it-dep')
    database.session.add(r)
    database.session.commit()
    return r


@pytest.fixture(scope="module")
def owner(users):
    """Community owner user."""
    return users['owner']


@pytest.fixture(scope="module")
def any_user(UserFixture, app, database):
    """A user without privileges or memberships."""
    u = UserFixture(
        email=f'anyuser@anyuser.org',
        password='anyuser',
    )
    u.create(app, database)
    u.identity  # compute identity
    return u


@pytest.fixture()
def admin(UserFixture, app, db, admin_role_need):
    """Admin user for requests."""
    u = UserFixture(
        email="admin@inveniosoftware.org",
        password="admin",
    )
    u.create(app, db)

    datastore = app.extensions["security"].datastore
    _, role = datastore._prepare_role_modify_args(u.user, "admin-access")

    datastore.add_role_to_user(u.user, role)
    datastore.commit()
    return u


@pytest.fixture()
def superuser_identity(admin, superuser_role_need):
    """Superuser identity fixture."""
    identity = admin.identity
    identity.provides.add(superuser_role_need)
    return identity


@pytest.fixture()
def admin_role_need(db):
    """Store 1 role with 'superuser-access' ActionNeed.

    WHY: This is needed because expansion of ActionNeed is
         done on the basis of a User/Role being associated with that Need.
         If no User/Role is associated with that Need (in the DB), the
         permission is expanded to an empty list.
    """
    role = Role(name="admin-access")
    db.session.add(role)

    action_role = ActionRoles.create(action=action_admin_access, role=role)
    db.session.add(action_role)

    db.session.commit()

    return action_role.need


@pytest.fixture()
def superuser_role_need(db):
    """Store 1 role with 'superuser-access' ActionNeed.
    WHY: This is needed because expansion of ActionNeed is
         done on the basis of a User/Role being associated with that Need.
         If no User/Role is associated with that Need (in the DB), the
         permission is expanded to an empty list.
    """
    role = Role(name="superuser-access")
    db.session.add(role)

    action_role = ActionRoles.create(action=superuser_access, role=role)
    db.session.add(action_role)

    db.session.commit()

    return action_role.need


#
# Communities
#
@pytest.fixture(scope="module")
def minimal_community():
    """Minimal community metadata."""
    return {
        "access": {
            "visibility": "public",
            "record_policy": "open",
        },
        "id": "public",
        "metadata": {
            "title": "My Community",
        }
    }


@pytest.fixture(scope="module")
def full_community():
    """Full community data as dict coming from the external world."""
    return {
        "access": {
            "visibility": "public",
            "member_policy": "open",
            "record_policy": "open",
        },
        "id": "my_community_id",
        "metadata": {
            "title": "My Community",
            "description": "This is an example Community.",
            "type": {
                "id": "event"
            },
            "curation_policy": "This is the kind of records we accept.",
            "page": "Information for my community.",
            "website": "https://inveniosoftware.org/",
            "organizations": [{
                    "name": "My Org",
            }]
        }
    }


@pytest.fixture(scope="module")
def community(community_service, owner, minimal_community, location):
    """A community."""
    c = community_service.create(owner.identity, minimal_community)
    owner.refresh()
    return c


@pytest.fixture(scope="module")
def restricted_community(community_service, owner, minimal_community, location):
    """A restricted community."""
    data = deepcopy(minimal_community)
    data['access']['visibility'] = 'restricted'
    data['id'] = 'restricted'
    c = community_service.create(owner.identity, data)
    owner.refresh()
    return c


@pytest.fixture(scope="function")
def fake_communities(
    community_service, owner, minimal_community, location, db,
    community_type_record, community_types
):
    data = deepcopy(minimal_community)
    """Multiple community created and posted to test search functionality."""
    N = 4
    for (type_, ind) in itertools.product(community_types, list(range(N))):
        data['id'] = f'comm_{type_["id"]}_{ind}'
        data['metadata']['type'] = {
            'id': type_['id']
        }
        c = community_service.create(owner.identity, data)
    Community.index.refresh()

    # Return ids of first and last created communities
    return 'comm_organization_0', 'comm_project_3', N, N*len(community_types)


#
# Community types
#
@pytest.fixture(scope="module")
def community_types():
    return [
        {"id": 'organization', "title": {"en": "Organization"}},
        {"id": 'event', "title": {"en": "Event"}},
        {"id": 'topic', "title": {"en": "Topic"}},
        {"id": 'project', "title": {"en": "Project"}}
    ]


@pytest.fixture()
def community_type_type(superuser_identity, vocabularies_service):
    """Creates and retrieves a language vocabulary type."""
    v = vocabularies_service.create_type(
        superuser_identity, "communitytypes", "comtyp")
    return v


@pytest.fixture(scope='module')
def community_types_data(community_types):
    """Example data."""
    return [
        {
            **ct,
            'type': 'communitytypes',
        } for ct in community_types
    ]


@pytest.fixture()
def community_type_record(
    superuser_identity, community_types_data, community_type_type,
    vocabularies_service
):
    """Creates a d retrieves community type records."""
    records_list = []
    for community_type_data in community_types_data:
        record = vocabularies_service.create(
            identity=superuser_identity,
            data=community_type_data
        )
        records_list.append(record)

    Vocabulary.index.refresh()  # Refresh the index
    return records_list



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community schema."""

from flask_babelex import lazy_gettext as _
from invenio_records_resources.services.records.schema import BaseRecordSchema
from invenio_vocabularies.contrib.affiliations.schema import \
    AffiliationRelationSchema
from invenio_vocabularies.contrib.awards.schema import FundingRelationSchema
from marshmallow import Schema, fields, validate
from marshmallow_utils.fields import NestedAttribute, SanitizedHTML, \
    SanitizedUnicode


def _not_blank(**kwargs):
    """Returns a non-blank validation rule."""
    max_ = kwargs.get('max','')
    return validate.Length(
        error=_('Field cannot be blank or longer than {max_} characters.'.format(max_=max_)),
        min=1,
        **kwargs
    )


# TODO: Move to Invenio-Records-Resources and make reusable (duplicated from
# Invenio-RDM-Records).
class Agent(Schema):
    """An agent schema."""

    user = fields.Integer(required=True)


class CommunityAccessSchema(Schema):

    owned_by = fields.List(fields.Nested(Agent))

    visibility = fields.Str(validate=validate.OneOf([
        'public',
        'restricted',
    ]))
    member_policy = fields.Str(validate=validate.OneOf([
        'open',
        'closed',
    ]))
    record_policy = fields.Str(validate=validate.OneOf([
        'open',
        'closed',
        'restricted',
    ]))


# TODO this schema is duplicated from invenio-rdm-records, can't be imported
# TODO in this module. Might need to be moved to invenio-vocabularies.
class VocabularySchema(Schema):
    """Invenio Vocabulary schema."""

    id = SanitizedUnicode(required=True)
    title = fields.Dict(dump_only=True)


class CommunityMetadataSchema(Schema):
    """Community metadata schema."""

    title = SanitizedUnicode(required=True, validate=_not_blank(max=250))
    description = SanitizedUnicode(validate=_not_blank(max=2000))

    curation_policy = SanitizedHTML(validate=_not_blank(max=2000))
    page = SanitizedHTML(validate=_not_blank(max=2000))

    type = fields.Nested(VocabularySchema)
    website = fields.Url(validate=_not_blank())
    funding = fields.List(fields.Nested(FundingRelationSchema))
    organizations = fields.List(fields.Nested(AffiliationRelationSchema))

    # TODO: Add when general vocabularies are ready
    # domains = fields.List(fields.Str())


class CommunitySchema(BaseRecordSchema):
    """Schema for the community metadata."""

    id = SanitizedUnicode(required=True, validate=_not_blank(max=100))
    uuid = fields.Method(serialize='get_uuid')
    metadata = NestedAttribute(CommunityMetadataSchema, required=True)
    access = NestedAttribute(CommunityAccessSchema, required=True)

    def get_uuid(self, obj):
        """Get the internal UUID."""
        return str(obj.id)


class CommunityFeaturedSchema(Schema):
    id = fields.Int(metadata={'read_only': True})
    start_date = fields.DateTime(required=True)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community access system field."""

from invenio_records.systemfields import SystemField
from .owners import Owners


class CommunityAccess:
    """Access management per community."""

    VISIBILITY_LEVELS = ('public', 'restricted')
    MEMBER_POLICY_LEVELS = ('open', 'closed')
    RECORD_POLICY_LEVELS = ('open', 'closed', 'restricted')

    owners_cls = Owners

    def __init__(
        self,
        visibility=None,
        member_policy=None,
        record_policy=None,
        owned_by=None,
        owners_cls=None,
    ):
        """Create a new CommunityAccess object.

        If ``owned_by`` is not specified, a new instance of ``owners_cls``
        will be used.

        :param visibility: The visibility level.
        :param owned_by: The set of community owners
        """
        self.visibility = visibility or 'public'
        self.member_policy = member_policy or 'open'
        self.record_policy = record_policy or 'open'

        owners_cls = owners_cls or self.owners_cls
        self.owned_by = owned_by if owned_by else owners_cls()

        self.errors = []

    def _validate_visibility_level(self, level):
        return level in self.VISIBILITY_LEVELS

    def _validate_member_policy_level(self, level):
        return level in self.MEMBER_POLICY_LEVELS

    def _validate_record_policy_level(self, level):
        return level in self.RECORD_POLICY_LEVELS

    @property
    def visibility(self):
        """Get the visibility level."""
        return self._visibility

    @visibility.setter
    def visibility(self, value):
        """Set the visibility level."""
        if not self._validate_visibility_level(value):
            raise ValueError(f"Unknown visibility level: {value}")
        self._visibility = value

    @property
    def member_policy(self):
        """Get the member policy level."""
        return self._member_policy

    @member_policy.setter
    def member_policy(self, value):
        """Set the member policy level."""
        if not self._validate_member_policy_level(value):
            raise ValueError(f"Unknown member policy level: {value}")
        self._member_policy = value

    @property
    def record_policy(self):
        """Get the record policy level."""
        return self._record_policy

    @record_policy.setter
    def record_policy(self, value):
        """Set the record policy level."""
        if not self._validate_record_policy_level(value):
            raise ValueError(f"Unknown record policy level: {value}")
        self._record_policy = value

    def dump(self):
        """Dump the field values as dictionary."""
        return {
            "visibility": self.visibility,
            "member_policy": self.member_policy,
            "record_policy": self.record_policy,
            "owned_by": self.owned_by.dump(),
        }

    def refresh_from_dict(self, access_dict):
        """Re-initialize the Access object with the data in the access_dict."""
        new_access = self.from_dict(access_dict)
        self.visibility = new_access.visibility
        self.member_policy = new_access.member_policy
        self.record_policy = new_access.record_policy
        self.owned_by = new_access.owned_by

    @classmethod
    def from_dict(
        cls,
        access_dict,
        owners_cls=None,
    ):
        """Create a new Access object from the specified 'access' property.

        The new ``CommunityAccess`` object will be populated with new instances
        from the configured classes.
        If ``access_dict`` is empty, the ``Access`` object will be populated
        with new instance ``owners_cls``.
        """
        owners_cls = owners_cls or cls.owners_cls

        errors = []

        # provide defaults in case there is no 'access' property
        owned_by = owners_cls()

        if access_dict:
            for owner_dict in access_dict.get("owned_by", []):
                try:
                    owned_by.add(owned_by.owner_cls(owner_dict))
                except Exception as e:
                    errors.append(e)

        access = cls(
            visibility=access_dict.get("visibility"),
            member_policy=access_dict.get("member_policy"),
            record_policy=access_dict.get("record_policy"),
            owned_by=owned_by
        )
        access.errors = errors
        return access

    def __repr__(self):
        """Return repr(self)."""
        return (
            "<{} (visibility: {}, "
            "member_policy: {}, "
            "record_policy: {}, "
            "owners: {})>"
        ).format(
            type(self).__name__,
            self.visibility,
            self.member_policy,
            self.record_policy,
            self.owned_by,
        )


# TODO: Move to Invenio-Records-Resources and make reusable. Duplicated from
# Invenio-RDM-Records.
class CommunityAccessField(SystemField):
    """System field for managing record access."""

    access_obj_class = CommunityAccess

    def __init__(self, key="access", access_obj_class=None):
        """Create a new RecordAccessField instance."""
        self._access_obj_class = access_obj_class or self.access_obj_class
        super().__init__(key=key)

    def _from_dict(self, instance, data):
        """Allows to override behavior in subclass."""
        return

    def obj(self, instance):
        """Get the access object."""
        obj = self._get_cache(instance)
        if obj is not None:
            return obj

        data = self.get_dictkey(instance)
        if data:
            obj = self._access_obj_class.from_dict(data)
        else:
            obj = self._access_obj_class()

        self._set_cache(instance, obj)
        return obj

    def set_obj(self, record, obj):
        """Set the access object."""
        # We accept both dicts and access class objects.
        if isinstance(obj, dict):
            obj = self._access_obj_class.from_dict(obj)

        assert isinstance(obj, self._access_obj_class)

        # We do not dump the object until the pre_commit hook
        # I.e. record.access != record['access']
        self._set_cache(record, obj)

    def __get__(self, record, owner=None):
        """Get the record's access object."""
        if record is None:
            # access by class
            return self

        # access by object
        return self.obj(record)

    def __set__(self, record, obj):
        """Set the records access object."""
        self.set_obj(record, obj)

    def pre_commit(self, record):
        """Dump the configured values before the record is committed."""
        obj = self.obj(record)
        if obj is not None:
            # only set the 'access' property if one was present in the
            # first place -- this was a problem in the unit test:
            # tests/resources/test_resources.py:test_simple_flow
            record["access"] = obj.dump()



"""Example of a record/community model."""

from invenio_db import db
from invenio_records.models import RecordMetadataBase

from invenio_communities.records.records.models import CommunityRelationMixin


class MockRecordMetadata(db.Model, RecordMetadataBase):
    """A baisc record."""

    __tablename__ = 'mock_metadata'


class MockRecordCommunity(db.Model, CommunityRelationMixin):
    """Relationship between record and community."""

    __tablename__ = 'mock_community'
    __record_model__ = MockRecordMetadata
    # __request_model__ = RequestMetadata


# class RecordRequest(db.Model, ReqestRelationMixin):
#     """Relationship between parent record and a request."""

#     __tablename__ = 'rdm_parents_request'
#     __record_model__ = RDMParentMetadata
#     # __request_model__ = RequestMetadata



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Command-line tools for demo module."""

import click
from faker import Faker
from flask.cli import with_appcontext
from invenio_access.permissions import system_identity
from invenio_communities.proxies import current_communities

from .fixtures.demo import create_fake_community
from .fixtures.tasks import create_demo_community


@click.group()
def communities():
    """Invenio communities commands."""


@communities.command('demo')
@with_appcontext
def demo():
    """Create 100 fake communities for demo purposes."""
    click.secho('Creating demo communities...', fg='green')
    faker = Faker()
    for _ in range(100):
        fake_data = create_fake_community(faker)
        create_demo_community.delay(fake_data)

    click.secho('Created communities!', fg='green')


@communities.command("rebuild-index")
@with_appcontext
def rebuild_index():
    click.secho("Reindexing communities...", fg="green")

    communities_service = current_communities.service
    communities_service.rebuild_index(identity=system_identity)

    click.secho("Reindexed communities!", fg="green")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Routes for community-related pages provided by Invenio-Communities."""

from flask import current_app, g, render_template
from flask_babelex import lazy_gettext as _
from flask_login import login_required
from invenio_access.permissions import system_identity
from invenio_records_resources.services.errors import PermissionDeniedError
from invenio_requests.proxies import current_requests_service
from invenio_vocabularies.proxies import current_service as vocabulary_service

from .decorators import pass_community, pass_community_logo


#
# Views
#

def communities_frontpage():
    """Communities index page."""
    return render_template(
        "invenio_communities/frontpage.html",
    )


def communities_search():
    """Communities search page."""
    return render_template(
        "invenio_communities/search.html",
    )


@login_required
def communities_new():
    """Communities creation page."""
    return render_template(
        "invenio_communities/new.html",
        form_config=dict(
            access=dict(
                visibility=[
                    {
                        'text': 'Public',
                        'value': 'public',
                        'icon': 'group',
                        'helpText': _('Your community is publicly accessible'
                                      ' and shows up in search results.')
                    },
                    {
                        'text': 'Restricted',
                        'value': 'restricted',
                        'icon': 'lock',
                        'helpText': _('Your community is restricted to users'
                                      ' with access.')
                    }
                ]
            ),
            SITE_UI_URL=current_app.config["SITE_UI_URL"]
        ),
    )


@pass_community
@pass_community_logo
def communities_settings(community=None, logo=None, pid_value=None):
    """Community settings/profile page."""
    permissions = community.has_permissions_to(
        ['update', 'read', 'search_requests', 'search_invites']
    )
    types = vocabulary_service.read_all(
        system_identity,
        fields=["id", "title"],
        type="communitytypes",
        max_records=10
    )
    comtypes = [{'id': i["id"], 'title': i["title"]} for i in list(types.hits)]
    if not permissions['can_update']:
        raise PermissionDeniedError()
    return render_template(
        "invenio_communities/details/settings/profile.html",
        community=community.to_dict(),  # TODO: use serializer,
        logo=logo.to_dict() if logo else None,
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        comtypes=comtypes,
        permissions=permissions,
        active_menu_tab="settings"
    )


@pass_community
@pass_community_logo
def communities_requests(community=None, logo=None, pid_value=None):
    """Community requests page."""
    permissions = community.has_permissions_to(
        ['update', 'read', 'search_requests', 'search_invites']
    )
    if not permissions['can_search_requests']:
        raise PermissionDeniedError()

    return render_template(
        "invenio_communities/details/requests/index.html",
        community=community.to_dict(),  # TODO: use serializer,
        logo=logo.to_dict() if logo else None,
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        permissions=permissions,
    )


@pass_community
@pass_community_logo
def communities_settings_privileges(community=None, logo=None, pid_value=None):
    """Community settings/privileges page."""
    permissions = community.has_permissions_to(
        ['update', 'read', 'search_requests', 'search_invites']
    )
    if not permissions['can_update']:
        raise PermissionDeniedError()
    return render_template(
        "invenio_communities/details/settings/privileges.html",
        community=community.to_dict(),  # TODO: use serializer,
        form_config=dict(
            access=dict(
                visibility=[
                    {
                        'text': 'Public',
                        'value': 'public',
                        'icon': 'group',
                        'helpText': _('Your community is publicly accessible'
                                      ' and shows up in search results.')
                    },
                    {
                        'text': 'Restricted',
                        'value': 'restricted',
                        'icon': 'lock',
                        'helpText': _('Your community is restricted to users'
                                      ' with access.')
                    }
                ]
            ),
        ),
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        permissions=permissions,
        logo=logo.to_dict() if logo else None
    )


@pass_community
def members(community=None, pid_value=None, endpoint=None):
    """Community members page."""
    permissions = community.has_permissions_to(
        [
            "update",
            "members_search",
            "members_search_public",
            "members_manage",
            "read",
            "search_requests",
            "search_invites",
            "invite_owners"
        ]
    )
    if not permissions['can_read']:
        raise PermissionDeniedError()
    return render_template(
        "invenio_communities/details/members/members.html",
        community=community.to_dict(),
        types={
            "organization": _("Organization"),
            "event": _("Event"),
            "topic": _("Topic"),
            "project": _("Project")
        },
        permissions=permissions,
    )


@pass_community
def invitations(community=None, pid_value=None):
    """Community invitations page."""
    permissions = community.has_permissions_to(
        ['update', 'read', 'search_requests', 'search_invites',
         'invite_owners']
    )
    if not permissions['can_search_invites']:
        raise PermissionDeniedError()
    return render_template(
        "invenio_communities/details/members/invitations.html",
        community=community.to_dict(),
        permissions=permissions,
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Northwestern University.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

from flask_babelex import gettext as _


"""Community base error."""


class CommunityError(Exception):
    """Base exception for community errors."""


class CommunityFeaturedEntryDoesNotExistError(CommunityError):
    """The provided set spec does not exist."""

    def __init__(self, query_arguments):
        """Initialise error."""
        super().__init__(_(f"A featured community entry with {query_arguments} does not exist."))
